from ._width import WidthValidator
from ._usecolormap import UsecolormapValidator
from ._show import ShowValidator
from ._project import ProjectValidator
from ._highlightwidth import HighlightwidthValidator
from ._highlightcolor import HighlightcolorValidator
from ._highlight import HighlightValidator
from ._color import ColorValidator
